# encoding: utf-8
# frozen_string_literal: true
require 'spec_helper'

#    The "Cc:" field (where the "Cc" means "Carbon Copy" in the sense of
#    making a copy on a typewriter using carbon paper) contains the
#    addresses of others who are to receive the message, though the
#    content of the message may not be directed at them.

describe Mail::CcField do
  
  describe "initialization" do

    it "should initialize" do
      expect { Mail::CcField.new("Mikel") }.not_to raise_error
    end

    it "should mix in the CommonAddress module" do
      expect(Mail::CcField.included_modules).to include(Mail::CommonAddress)
    end

    it "should accept a string without the field name" do
      t = Mail::CcField.new('Mikel Lindsaar <mikel@test.lindsaar.net>, "Bob Smith" <bob@me.com>')
      expect(t.name).to eq 'Cc'
      expect(t.value).to eq 'Mikel Lindsaar <mikel@test.lindsaar.net>, "Bob Smith" <bob@me.com>'
    end

  end
  
  # Actual testing of CommonAddress methods occurs in the address field spec file

  describe "instance methods" do
    it "should return an address" do
      t = Mail::CcField.new('Mikel Lindsaar <mikel@test.lindsaar.net>')
      expect(t.formatted).to eq ['Mikel Lindsaar <mikel@test.lindsaar.net>']
    end

    it "should return two addresses" do
      t = Mail::CcField.new('Mikel Lindsaar <mikel@test.lindsaar.net>, Ada Lindsaar <ada@test.lindsaar.net>')
      expect(t.formatted.first).to eq 'Mikel Lindsaar <mikel@test.lindsaar.net>'
      expect(t.addresses.last).to eq 'ada@test.lindsaar.net'
    end

    it "should return one address and a group" do
      t = Mail::CcField.new('sam@me.com, my_group: mikel@me.com, bob@you.com;')
      expect(t.addresses[0]).to eq 'sam@me.com'
      expect(t.addresses[1]).to eq 'mikel@me.com'
      expect(t.addresses[2]).to eq 'bob@you.com'
    end
    
    it "should return the formatted line on to_s" do
      t = Mail::CcField.new('sam@me.com, my_group: mikel@me.com, bob@you.com;')
      expect(t.value).to eq 'sam@me.com, my_group: mikel@me.com, bob@you.com;'
    end
    
    it "should return the encoded line for one address" do
      t = Mail::CcField.new('sam@me.com')
      expect(t.encoded).to eq "Cc: sam@me.com\r\n"
    end
    
    it "should return the encoded line" do
      t = Mail::CcField.new('sam@me.com, my_group: mikel@me.com, bob@you.com;')
      expect(t.encoded).to eq "Cc: sam@me.com, \r\n\smy_group: mikel@me.com, \r\n\sbob@you.com;\r\n"
    end
    
    it "should return the decoded line" do
      t = Mail::CcField.new('sam@me.com, my_group: mikel@me.com, bob@you.com;')
      expect(t.decoded).to eq "sam@me.com, my_group: mikel@me.com, bob@you.com;"
    end
    
  end
  
  
end
